@@ -328,6 +328,28 @@ static const uint32_t pp_yuv420p8_scaling_gen8[][4] = {
 #include "shaders/post_processing/gen8/conv_nv12.g8b"
 };
 
+static const uint32_t pp_8bit_420_rgb32_scaling_gen8[][4] = {
+#include "shaders/post_processing/gen8/conv_8bit_420_rgb32.g8b"
+};
+
+struct i965_kernel pp_common_scaling_gen8[] = {
+    {
+        "8bit to 8bit",
+        0,
+        pp_yuv420p8_scaling_gen8,
+        sizeof(pp_yuv420p8_scaling_gen8),
+        NULL,
+    },
+
+    {
+        "8bit 420 to rgb32",
+        1,
+        pp_8bit_420_rgb32_scaling_gen8,
+        sizeof(pp_8bit_420_rgb32_scaling_gen8),
+        NULL,
+    },
+};
+
 static void
 gen8_pp_set_surface_tiling(struct gen8_surface_state *ss, unsigned int tiling)
 {
@@ -388,7 +410,8 @@ gen8_pp_set_surface_state(VADriverContextP ctx, struct i965_post_processing_cont
     ss = (struct gen8_surface_state *)((char *)ss_bo->virtual + SURFACE_STATE_OFFSET(index));
     memset(ss, 0, sizeof(*ss));
 
-    if (IS_GEN9(i965->intel.device_info))
+    if (IS_GEN9(i965->intel.device_info) ||
+        IS_GEN10(i965->intel.device_info))
         ss->ss1.surface_mocs = GEN9_CACHE_PTE;
 
     ss->ss0.surface_type = I965_SURFACE_2D;
@@ -437,7 +460,8 @@ gen8_pp_set_surface2_state(VADriverContextP ctx, struct i965_post_processing_con
     ss2 = (struct gen8_surface_state2 *)((char *)ss2_bo->virtual + SURFACE_STATE_OFFSET(index));
     memset(ss2, 0, sizeof(*ss2));
 
-    if (IS_GEN9(i965->intel.device_info))
+    if (IS_GEN9(i965->intel.device_info) ||
+        IS_GEN10(i965->intel.device_info))
         ss2->ss5.surface_object_mocs = GEN9_CACHE_PTE;
 
     ss2->ss6.base_addr = surf_bo->offset + surf_bo_offset;
@@ -1661,7 +1685,6 @@ gen8_post_processing_context_init(VADriverContextP ctx,
     struct i965_driver_data *i965 = i965_driver_data(ctx);
     struct i965_post_processing_context *pp_context = data;
     struct i965_gpe_context *gpe_context;
-    struct i965_kernel scaling_kernel;
 
     gen8_post_processing_context_common_init(ctx, data, pp_modules_gen8, ARRAY_ELEMS(pp_modules_gen8), batch);
     avs_init_state(&pp_context->pp_avs_context.state, &gen8_avs_config);
@@ -1673,12 +1696,9 @@ gen8_post_processing_context_init(VADriverContextP ctx,
      * I420 ->NV12
      */
     gpe_context = &pp_context->scaling_gpe_context;
-    memset(&scaling_kernel, 0, sizeof(scaling_kernel));
-    scaling_kernel.bin = pp_yuv420p8_scaling_gen8;
-    scaling_kernel.size = sizeof(pp_yuv420p8_scaling_gen8);
-    gen8_gpe_load_kernels(ctx, gpe_context, &scaling_kernel, 1);
+    gen8_gpe_load_kernels(ctx, gpe_context, pp_common_scaling_gen8, ARRAY_ELEMS(pp_common_scaling_gen8));
     gpe_context->idrt.entry_size = ALIGN(sizeof(struct gen8_interface_descriptor_data), 64);
-    gpe_context->idrt.max_entries = 1;
+    gpe_context->idrt.max_entries = ALIGN(ARRAY_ELEMS(pp_common_scaling_gen8), 2);
     gpe_context->sampler.entry_size = ALIGN(sizeof(struct gen8_sampler_state), 64);
     gpe_context->sampler.max_entries = 1;
     gpe_context->curbe.length = ALIGN(sizeof(struct scaling_input_parameter), 32);
@@ -1707,7 +1727,7 @@ gen8_post_processing_context_init(VADriverContextP ctx,
     gpe_context->vfe_state.gpgpu_mode = 0;
 
     gen8_gpe_context_init(ctx, gpe_context);
-    pp_context->scaling_gpe_context_initialized |= VPPGPE_8BIT_8BIT;
+    pp_context->scaling_gpe_context_initialized |= (VPPGPE_8BIT_8BIT | VPPGPE_8BIT_420_RGB32);
 
     return;
 }
@@ -1854,13 +1874,13 @@ gen8_gpe_context_yuv420p8_scaling_curbe(VADriverContextP ctx,
 
     fourcc = pp_get_surface_fourcc(ctx, src_surface);
     if (fourcc == VA_FOURCC_NV12) {
-        scaling_curbe->dw7.src_packed = 1;
+        scaling_curbe->dw2.src_packed = 1;
     }
 
     fourcc = pp_get_surface_fourcc(ctx, dst_surface);
 
     if (fourcc == VA_FOURCC_NV12) {
-        scaling_curbe->dw7.dst_packed = 1;
+        scaling_curbe->dw2.dst_packed = 1;
     }
 
     i965_gpe_context_unmap_curbe(gpe_context);
@@ -1892,13 +1912,17 @@ gen8_pp_context_get_surface_conf(VADriverContextP ctx,
         pitch[0] = obj_surface->width;
         bo_offset[0] = 0;
 
-        if (fourcc == VA_FOURCC_P010 || fourcc == VA_FOURCC_NV12) {
+        if (fourcc == VA_FOURCC_RGBX ||
+            fourcc == VA_FOURCC_RGBA ||
+            fourcc == VA_FOURCC_BGRX ||
+            fourcc == VA_FOURCC_BGRA) {
+            /* nothing to do here */
+        } else if (fourcc == VA_FOURCC_P010 || fourcc == VA_FOURCC_NV12) {
             width[1] = width[0] / 2;
             height[1] = height[0] / 2;
             pitch[1] = obj_surface->cb_cr_pitch;
             bo_offset[1] = obj_surface->width * obj_surface->y_cb_offset;
         } else {
-            /* I010/I420 format */
             width[1] = width[0] / 2;
             height[1] = height[0] / 2;
             pitch[1] = obj_surface->cb_cr_pitch;
@@ -1919,22 +1943,30 @@ gen8_pp_context_get_surface_conf(VADriverContextP ctx,
         pitch[0] = obj_image->image.pitches[0];
         bo_offset[0] = obj_image->image.offsets[0];
 
-        if (fourcc == VA_FOURCC_P010 || fourcc == VA_FOURCC_NV12) {
+        if (fourcc == VA_FOURCC_RGBX ||
+            fourcc == VA_FOURCC_RGBA ||
+            fourcc == VA_FOURCC_BGRX ||
+            fourcc == VA_FOURCC_BGRA) {
+            /* nothing to do here */
+        } else if (fourcc == VA_FOURCC_P010 || fourcc == VA_FOURCC_NV12) {
             width[1] = width[0] / 2;
             height[1] = height[0] / 2;
             pitch[1] = obj_image->image.pitches[1];
             bo_offset[1] = obj_image->image.offsets[1];
         } else {
-            /* I010/I420 format */
-            /* YV12 is TBD */
+            int u = 1, v = 2;
+
+            if (fourcc == VA_FOURCC_YV12 || fourcc == VA_FOURCC_IMC1)
+                u = 2, v = 1;
+
             width[1] = width[0] / 2;
             height[1] = height[0] / 2;
-            pitch[1] = obj_image->image.pitches[1];
-            bo_offset[1] = obj_image->image.offsets[1];
+            pitch[1] = obj_image->image.pitches[u];
+            bo_offset[1] = obj_image->image.offsets[u];
             width[2] = width[0] / 2;
             height[2] = height[0] / 2;
-            pitch[2] = obj_image->image.pitches[2];
-            bo_offset[2] = obj_image->image.offsets[2];
+            pitch[2] = obj_image->image.pitches[v];
+            bo_offset[2] = obj_image->image.offsets[v];
         }
 
     }
@@ -2105,3 +2137,251 @@ gen8_yuv420p8_scaling_post_processing(
 
     return VA_STATUS_SUCCESS;
 }
+static void
+gen8_gpe_context_8bit_420_rgb32_scaling_curbe(VADriverContextP ctx,
+                                              struct i965_gpe_context *gpe_context,
+                                              VARectangle *src_rect,
+                                              struct i965_surface *src_surface,
+                                              VARectangle *dst_rect,
+                                              struct i965_surface *dst_surface)
+{
+    struct scaling_input_parameter *scaling_curbe;
+    float src_width, src_height;
+    float coeff;
+    unsigned int fourcc;
+    int src_format = SRC_FORMAT_I420, dst_format = DST_FORMAT_RGBX;
+    const float * yuv_to_rgb_coefs;
+    size_t yuv_to_rgb_coefs_size;
+
+    if ((gpe_context == NULL) ||
+        (src_rect == NULL) || (src_surface == NULL) ||
+        (dst_rect == NULL) || (dst_surface == NULL))
+        return;
+
+    scaling_curbe = i965_gpe_context_map_curbe(gpe_context);
+
+    if (!scaling_curbe)
+        return;
+
+    memset(scaling_curbe, 0, sizeof(struct scaling_input_parameter));
+
+    scaling_curbe->bti_input = BTI_SCALING_INPUT_Y;
+    scaling_curbe->bti_output = BTI_SCALING_OUTPUT_Y;
+
+    /* As the src_rect/dst_rect is already checked, it is skipped.*/
+    scaling_curbe->x_dst     = dst_rect->x;
+    scaling_curbe->y_dst     = dst_rect->y;
+
+    src_width = src_rect->x + src_rect->width;
+    src_height = src_rect->y + src_rect->height;
+
+    scaling_curbe->inv_width = 1 / src_width;
+    scaling_curbe->inv_height = 1 / src_height;
+
+    coeff = (float)(src_rect->width) / dst_rect->width;
+    scaling_curbe->x_factor = coeff / src_width;
+    scaling_curbe->x_orig = (float)(src_rect->x) / src_width;
+
+    coeff = (float)(src_rect->height) / dst_rect->height;
+    scaling_curbe->y_factor = coeff / src_height;
+    scaling_curbe->y_orig = (float)(src_rect->y) / src_height;
+
+    fourcc = pp_get_surface_fourcc(ctx, src_surface);
+
+    switch (fourcc) {
+    case VA_FOURCC_I420:
+    case VA_FOURCC_IMC3: /* pitch / base address is set via surface_state */
+        src_format = SRC_FORMAT_I420;
+        break;
+
+    case VA_FOURCC_NV12:
+        src_format = SRC_FORMAT_NV12;
+        break;
+
+    case VA_FOURCC_YV12:
+    case VA_FOURCC_IMC1: /* pitch / base address is set via surface_state */
+        src_format = SRC_FORMAT_YV12;
+        break;
+
+    default:
+        break;
+    }
+
+    fourcc = pp_get_surface_fourcc(ctx, dst_surface);
+
+    switch (fourcc) {
+    case VA_FOURCC_RGBX:
+        dst_format = DST_FORMAT_RGBX;
+        break;
+
+    case VA_FOURCC_RGBA:
+        dst_format = DST_FORMAT_RGBA;
+        break;
+
+    case VA_FOURCC_BGRX:
+        dst_format = DST_FORMAT_BGRX;
+        break;
+
+    case VA_FOURCC_BGRA:
+        dst_format = DST_FORMAT_BGRA;
+        break;
+
+    default:
+        break;
+    }
+
+    scaling_curbe->dw2.src_format = src_format;
+    scaling_curbe->dw2.dst_format = dst_format;
+
+    yuv_to_rgb_coefs = i915_color_standard_to_coefs(i915_filter_to_color_standard(src_surface->flags & VA_SRC_COLOR_MASK), &yuv_to_rgb_coefs_size);
+    memcpy(&scaling_curbe->coef_ry, yuv_to_rgb_coefs, yuv_to_rgb_coefs_size);
+
+    i965_gpe_context_unmap_curbe(gpe_context);
+}
+
+static void
+gen8_gpe_context_8bit_420_rgb32_scaling_surfaces(VADriverContextP ctx,
+                                                 struct i965_gpe_context *gpe_context,
+                                                 VARectangle *src_rect,
+                                                 struct i965_surface *src_surface,
+                                                 VARectangle *dst_rect,
+                                                 struct i965_surface *dst_surface)
+{
+    unsigned int fourcc;
+    int width[3], height[3], pitch[3], bo_offset[3];
+    dri_bo *bo;
+    struct object_surface *obj_surface;
+    struct object_image *obj_image;
+    int bti;
+
+    if ((gpe_context == NULL) ||
+        (src_rect == NULL) || (src_surface == NULL) ||
+        (dst_rect == NULL) || (dst_surface == NULL))
+        return;
+
+    if (src_surface->base == NULL || dst_surface->base == NULL)
+        return;
+
+    fourcc = pp_get_surface_fourcc(ctx, src_surface);
+
+    if (src_surface->type == I965_SURFACE_TYPE_SURFACE) {
+        obj_surface = (struct object_surface *)src_surface->base;
+        bo = obj_surface->bo;
+    } else {
+        obj_image = (struct object_image *)src_surface->base;
+        bo = obj_image->bo;
+    }
+
+    if (gen8_pp_context_get_surface_conf(ctx, src_surface, src_rect,
+                                         width, height, pitch,
+                                         bo_offset)) {
+        /* Input surface */
+        bti = BTI_SCALING_INPUT_Y;
+        gen8_add_dri_buffer_2d_gpe_surface(ctx, gpe_context, bo,
+                                           bo_offset[0],
+                                           width[0], height[0],
+                                           pitch[0], 0,
+                                           I965_SURFACEFORMAT_R8_UNORM,
+                                           bti, 0);
+
+        if (fourcc == VA_FOURCC_NV12) {
+            gen8_add_dri_buffer_2d_gpe_surface(ctx, gpe_context, bo,
+                                               bo_offset[1],
+                                               width[1], height[1],
+                                               pitch[1], 0,
+                                               I965_SURFACEFORMAT_R8G8_UNORM,
+                                               bti + 1, 0);
+        } else {
+            /* The corresponding shader handles U, V plane in order */
+            gen8_add_dri_buffer_2d_gpe_surface(ctx, gpe_context, bo,
+                                               bo_offset[1],
+                                               width[1], height[1],
+                                               pitch[1], 0,
+                                               I965_SURFACEFORMAT_R8_UNORM,
+                                               bti + 1, 0);
+
+            gen8_add_dri_buffer_2d_gpe_surface(ctx, gpe_context, bo,
+                                               bo_offset[2],
+                                               width[2], height[2],
+                                               pitch[2], 0,
+                                               I965_SURFACEFORMAT_R8_UNORM,
+                                               bti + 2, 0);
+        }
+    }
+
+    fourcc = pp_get_surface_fourcc(ctx, dst_surface);
+
+    if (dst_surface->type == I965_SURFACE_TYPE_SURFACE) {
+        obj_surface = (struct object_surface *)dst_surface->base;
+        bo = obj_surface->bo;
+    } else {
+        obj_image = (struct object_image *)dst_surface->base;
+        bo = obj_image->bo;
+    }
+
+    if (gen8_pp_context_get_surface_conf(ctx, dst_surface, dst_rect,
+                                         width, height, pitch,
+                                         bo_offset)) {
+        assert(fourcc == VA_FOURCC_RGBX ||
+               fourcc == VA_FOURCC_RGBA ||
+               fourcc == VA_FOURCC_BGRX ||
+               fourcc == VA_FOURCC_BGRA);
+        assert(width[0] * 4 <= pitch[0]);
+
+        /* output surface */
+        bti = BTI_SCALING_OUTPUT_Y;
+        gen8_add_dri_buffer_2d_gpe_surface(ctx, gpe_context, bo,
+                                           bo_offset[0],
+                                           width[0] * 4, height[0],
+                                           pitch[0], 1,
+                                           I965_SURFACEFORMAT_R8_UINT,
+                                           bti, 0);
+    }
+}
+
+VAStatus
+gen8_8bit_420_rgb32_scaling_post_processing(VADriverContextP   ctx,
+                                            struct i965_post_processing_context *pp_context,
+                                            struct i965_surface *src_surface,
+                                            VARectangle *src_rect,
+                                            struct i965_surface *dst_surface,
+                                            VARectangle *dst_rect)
+{
+    struct i965_gpe_context *gpe_context;
+    struct gpe_media_object_walker_parameter media_object_walker_param;
+    struct intel_vpp_kernel_walker_parameter kernel_walker_param;
+
+    if (!pp_context || !src_surface || !src_rect || !dst_surface || !dst_rect)
+        return VA_STATUS_ERROR_INVALID_PARAMETER;
+
+    if (!(pp_context->scaling_gpe_context_initialized & VPPGPE_8BIT_420_RGB32))
+        return VA_STATUS_ERROR_UNIMPLEMENTED;
+
+    gpe_context = &pp_context->scaling_gpe_context;
+
+    gen8_gpe_context_init(ctx, gpe_context);
+    gen8_vpp_scaling_sample_state(ctx, gpe_context, src_rect, dst_rect);
+    gen8_gpe_reset_binding_table(ctx, gpe_context);
+    gen8_gpe_context_8bit_420_rgb32_scaling_curbe(ctx, gpe_context,
+                                                  src_rect, src_surface,
+                                                  dst_rect, dst_surface);
+
+    gen8_gpe_context_8bit_420_rgb32_scaling_surfaces(ctx, gpe_context,
+                                                     src_rect, src_surface,
+                                                     dst_rect, dst_surface);
+
+    gen8_gpe_setup_interface_data(ctx, gpe_context);
+
+    memset(&kernel_walker_param, 0, sizeof(kernel_walker_param));
+    kernel_walker_param.resolution_x = ALIGN(dst_rect->width, 16) >> 4;
+    kernel_walker_param.resolution_y = ALIGN(dst_rect->height, 16) >> 4;
+    kernel_walker_param.no_dependency = 1;
+
+    intel_vpp_init_media_object_walker_parameter(&kernel_walker_param, &media_object_walker_param);
+    media_object_walker_param.interface_offset = 1;
+    gen8_run_kernel_media_object_walker(ctx, pp_context->batch,
+                                        gpe_context,
+                                        &media_object_walker_param);
+
+    return VA_STATUS_SUCCESS;
+}
